#!/bin/bash

function show_usage ()
{
	echo $"Usage: $0 /path/to/kernel/source /path/to/module/source /path/to/build/dir kernel-version"
	exit 1
}

# If invoked as pre_build.sh --getsource <kernel-version> then the script attempts to copy the
# kernel sourcecode files needed for the build into the current directory
if [ $# = 2 -a "X$1" = "X--getsource" ]; then
	basekernel=`echo $2|sed 's/-.*//g'`
	rpmbuilddir=`rpm --eval '%_builddir'`
	if [ -d $rpmbuilddir/kernel-$basekernel/linux-$basekernel ]; then
		set -- $rpmbuilddir/kernel-$basekernel/linux-$basekernel `dirname $0` . "$2"
	else
		set -- $rpmbuilddir/kernel-$basekernel/linux-$basekernel.`arch` `dirname $0` . "$2"
	fi
	unset basekernel
fi

[ $# != 4 ] && show_usage;

kernel_source_dir=$1
module_source_dir=$2
build_dir=$3
kernel_version=$4

VERSION_WO_SUFFIX=`echo ${kernel_version} | sed -e 's/smp\|BOOT\|bigmem\|hugemem\|enterprise\|debug//'`
# Apply a different patch set for RHEL kernels
if echo $VERSION_WO_SUFFIX | grep -Fi .EL > /dev/null; then
	ENTERPRISE_KERNEL=yes
	BASE_KERNEL=2.6.9-5.0.3.EL
else
	ENTERPRISE_KERNEL=no
	BASE_KERNEL=2.6.5-1.358
fi
# Files needed to build Advansys driver
ADVANSYS_FILE_LIST="advansys.c advansys.h hosts.h scsi.h scsi_module.c scsi_obsolete.h scsi_typedefs.h"
# Files needed to build scsi_mod module to get dependency right
SCSIMOD_FILE_LIST="constants.c hosts.c scsi.c scsicam.c scsi_devinfo.c scsi_error.c scsi_ioctl.c
	scsi_lib.c scsi_logging.h scsi_priv.h scsi_sysfs.c scsi_scan.c scsi_syms.c"

# In order to keep the size of the source tree down, yet still support multiple kernel
# versions, we start with the kernel source for the standard FC2 install kernel (2.6.5-1.358)
# or RHEL4 kernel (2.6.9-5.0.3.EL) and then apply zero or more patches to bring it up to date.
#
# None of this is necessary though if the kernel-sourcecode RPM is installed, as we just use
# the source from there if it's available.

# See if the kernel source appears to be present
if [ -f ${kernel_source_dir}/drivers/scsi/advansys.c -a \
     -f ${kernel_source_dir}/drivers/scsi/advansys.h ]; then
	# Yes; copy in files from the kernel source area
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		if [ -f ${kernel_source_dir}/drivers/scsi/$f ]; then
			cp ${kernel_source_dir}/drivers/scsi/$f $build_dir
			echo "Using ${kernel_source_dir}/drivers/scsi/$f"
		else
			echo "$f not present in kernel source, skipping"
		fi
	done
elif [ -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/advansys.c -a \
       -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/advansys.h ]; then
	# Yes; copy in files from the kernel source area
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		if [ -f /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f ]; then
			cp /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f $build_dir
			echo "Using /usr/src/linux-$VERSION_WO_SUFFIX/drivers/scsi/$f"
		else
			echo "$f not present in kernel source, skipping"
		fi
	done
else
	# No, use our own copy of the source
	for f in $ADVANSYS_FILE_LIST $SCSIMOD_FILE_LIST
	do
		cp $module_source_dir/kernel-source/$BASE_KERNEL/$f $build_dir
		echo "Using $module_source_dir/kernel-source/$BASE_KERNEL/$f"
	done
	# Create list of patches needed to bring source up to date
	PATCHLIST=`echo $VERSION_WO_SUFFIX |
		case $ENTERPRISE_KERNEL in
		no)	awk '	/^2\.6\.[0-5]-/		{ done = 1; exit }
				/^2\.6\.[0-9]+-/	{
							  # 2.6.6-
							  print "2.6.6-1.427-advansys.patch" }
				/^2\.6\.([7-9]|[1-9][0-9]+)-/ {
							  # 2.6.7-
							  print "2.6.7-1.494.2.2-advansys.patch" }
				/^2\.6\.([89]|[1-9][0-9]+)-/ {
							  # 2.6.8-* (except 2.6.8-1.521)
							  if ($1 == "2.6.8-1.521") { next }
							  print "2.6.8-1.541-advansys.patch" }
				/^2\.6\.(9|[1-9][0-9]+)-/ {
							  # 2.6.9-*
							  print "2.6.9-1.667-advansys.patch" }
				/^2\.6\.(9|[1-9][0-9]+)-/ {
							  # 2.6.9-* >= 1.700
							  if ($1 ~ "2.6.9-1.[0-6][0-9][0-9]") { next }
							  print "2.6.9-1.724-advansys.patch" }
				/^2\.6\.([1-9][0-9]+)-/ {
							  # 2.6.10-*
							  print "2.6.10-1.737-advansys.patch" }'
			;;
		*)	awk '	/^2\.6\.9-([0-9]|1[0-9]|21)\..*\.EL/ {
							  # 2.6.9- 0 to 21 (RHEL 4.0 to 4.1)
							  done = 1; exit }
				/^2\.6\.9-(2[2-9]|[3-9][0-9]|[1-9][0-9][0-9])\..*\.EL/ {
							  # 2.6.9-22 onwards (RHEL 4.2)
							  print "2.6.9-22.EL-advansys.patch" }
				/^2\.6\.9-(3[4-9]|[4-9][0-9]|[1-9][0-9][0-9])\..*\.EL/ {
							  # 2.6.9-34 onwards (RHEL 4.3)
							  print "2.6.9-34.EL-advansys.patch" }
				/^2\.6\.9-(4[2-9]|[5-9][0-9]|[1-9][0-9][0-9])\..*\.EL/ {
							  # 2.6.9-42 onwards (RHEL 4.4)
							  print "2.6.9-42.EL-advansys.patch" }
				/^2\.6\.9-(5[5-9]|[6-9][0-9]|[1-9][0-9][0-9])\..*\.EL/ {
							  # 2.6.9-55 onwards (RHEL 4.5)
							  print "2.6.9-55.EL-advansys.patch" }
				/^2\.6\.18-/ {
							  # 2.6.18 onwards (RHEL 5.0)
							  print "2.6.18-8.el5-advansys.patch" }
				/^2\.6\.18-(5[3-9]|[6-9][0-9]|[1-9][0-9][0-9])\..*\.el/ {
							  # 2.6.18-53 onwards (RHEL 5.1)
							  print "2.6.18-53.el5-advansys.patch" }'
			;;
		esac`
	if [ -n "$PATCHLIST" ]; then
		cd $build_dir
		for p in $PATCHLIST
		do
			echo "Applying patch $p"
			patch -p1 --quiet < $module_source_dir/kernel-source/$p
		done
	fi
fi
